package de.uni_potsdam.hpi.gds.digisim.plugin;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;

import de.ksquared.jds.components.buildin.flipflop.DFlipFlop;
import de.ksquared.jds.components.buildin.flipflop.FlipFlop;
import de.ksquared.jds.contacts.Contact;
import de.ksquared.jds.contacts.ContactUtilities;
import de.ksquared.jds.contacts.InputContact;
import de.ksquared.jds.contacts.OutputContact;

@FlipFlop.Clocked
@FlipFlop.MasterSlave

public class ResetDFlipFlop extends DFlipFlop
{

	private static final long serialVersionUID = -5865477862350360785L;

	private InputContact reset;
	
	private OutputContact q;
	private OutputContact nq;
	
	private Contact[] contacts;
	
	public static final ComponentAttributes componentAttributes = 
			new ComponentAttributes("D-Flip-Flop mit Reset",
											"group.flipflop",
											"D-Flip-Flop mit Reset",
											"Daniel Kurzynski",
											3);
	
	public ResetDFlipFlop()
	{
		
		int top = this.getSize().height / 4;
		
		this.reset = new InputContact(this, new Point(0, (int)(top * 3.5)));
		
		q= (OutputContact) super.getContacts()[2];
		nq= (OutputContact) super.getContacts()[3];
		
		this.contacts = ContactUtilities.concatenateContacts(reset, super.getContacts());
		
	}
	
	@Override
	public Contact[] getContacts()
	{
		return contacts;
		
	}
	
	@Override
	public void paint(Graphics graphics)
	{
		super.paint(graphics);
		
		int top = this.getSize().height / 4;
		
		ContactUtilities.paintSolderingJoint(graphics, 10, 0, this.reset);
		
		graphics.drawString("R", 15, (int)(top * 3.5));
		
	}

	@Override
	 public void calculate() { 
		
		if(this.reset.isCharged())
		{
			q.setCharged(false);
			nq.setCharged(true);
		}
		
		super.calculate();
		
	}

	@Override
	public Dimension getSize()
	{
		return super.getSize();
	}


	
}
