package de.uni_potsdam.hpi.gds.digisim.plugin;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.beans.PropertyVetoException;
import java.util.HashMap;
import java.util.Map;

import de.ksquared.jds.components.Component;
import de.ksquared.jds.components.Configurable;
import de.ksquared.jds.components.Configurable.Option.OptionType;
import de.ksquared.jds.components.Sociable;
import de.ksquared.jds.contacts.Contact;
import de.ksquared.jds.contacts.InputContact;
import de.ksquared.jds.contacts.OutputContact;

public class SwitchOnDelay extends Component implements Sociable, Configurable
{
	private static final long serialVersionUID = 7410944824193118863L;

	public static final ComponentAttributes componentAttributes = 
			new ComponentAttributes(
					"Einschaltverzögerung", 
					"group.others", 
					"Einschaltverzögerung",
					"Daniel Kurzynski", 1);
	
	private InputContact input;
	private OutputContact output;
	
	private Dimension size = new Dimension(50, 50);
	
	private int delay = 1;
	private transient int clock =0;
	
	public SwitchOnDelay()
	{
		input = new InputContact(this, new Point(0,size.height/2));

		output = new OutputContact(this, new Point(size.width,size.height/2));
		
	}
	
	@Override
	public Contact[] getContacts()
	{
		
		return new Contact[] { input, output };
	}

	@Override
	public void calculate()
	{
		if(input.isCharged())
		{
			if(this.clock++ <= this.delay)
				output.setCharged(false);
			else
			{
				this.clock = this.delay+1;
				output.setCharged();
			}
		}
		else
		{
			output.setCharged(false);
			this.clock = 0;
		}
		
	}

	@Override
	public Dimension getSize()
	{
		return size;
	}
	
	@Override
	public void paint(Graphics graphics)
	{
		graphics.setColor(Color.BLACK);
		
		graphics.drawRect(5, 5, 
				size.width-10, 
				size.height-10);
		
		graphics.drawString("0 - τ", 12, 20);
	}

	@Override
	public Map<Option, Object> getConfiguration()
	{
		Option[] options = getOptions();
		Map<Option, Object> configuration = new HashMap<Option, Object>();
		configuration.put(options[0], delay);
		return configuration;
	}

	@Override
	public Option[] getOptions()
	{
		return new Option[] { 
				new Option("delay", "Verzögerung", OptionType.NUMBER, 1)};
	}

	@Override
	public void setConfiguration(Map<Option, Object> configuration)
			throws PropertyVetoException
	{
		Option[] options = getOptions();
		int newDelay = (Integer) configuration.get(options[0]);
		
		if (newDelay <= 0)
			throw new PropertyVetoException(
				"Die Verzögerung muss positiv sein", null);
		
		this.delay = newDelay;
		this.clock = 0;
		
	}
}
